/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, Rolf.Dubitzky@gmx.net
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

// STDC
#include <unistd.h>
#include <getopt.h>

// STDC++
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
using std::string;
#include <cassert>

// PIAVE
#include "libpiave/piave_base.hh"
#include "libpiave/mediaelements.hh"
#include "libpiave/exception.hh"
#include "libpiave/property.hh"
#include "libpiave/plugin.hh"
#include "libpiave/pluginmgr.hh"
#include "libpiave/streammgr.hh"

using namespace PIAVE;

void usage( int argc,  char * const * argv  ) { 
    cout << "this tool is supposed to read a XML based specification of "
        "input files / effect tree and target format in VEML format. "
        "but that is not yet possible. For now, you can only write out "
        "what you read in. Not very useful, but it's good for testing." << endl;
    cout << "usage:  "<<argv[0]<<" [options] <ipnut file> <output file>"<<endl;
    cout << " -v <level>       level of verbosity (default: 0=quiet) " << endl;
    cout << " -d <dir>         search <dir> recursively for plugins" << endl;
    cout << " -l <lib>         load plugin library <lib>" << endl;
    cout << " -n <sec>         number of seconds to encode" << endl;
    exit( 1 ); 
}

int
main( int argc,  char * const * argv  )
{

    bool    default_plugin_src   = true;
    int     n_plugins            = 0;
    Time    n_seconds( -1 );

    int c;
    opterr = 0;
    while ((c = getopt (argc, argv, "d:l:n:v:qh")) != -1) {
        switch (c)
        {
          case 'v':
              PIAVE::Global::verbosity = atoi( optarg );
              break;
          case 'h':
              usage( argc, argv );
              break;
          case 'd':
              n_plugins += PluginMgr::openDLLsFromDir( optarg );
              default_plugin_src = false;
              break;
          case 'n':
              n_seconds = Time( atoi( optarg ) );
              break;
          case 'l':
              n_plugins += PluginMgr::openDLL( optarg ); 
              default_plugin_src = false;
              break;
          default:
              cout <<"unknown option: "<<(char)c<<endl;
              usage( argc, argv );
              break;
        }
    }

    if ( optind > argc-2 ) {
        cout << "ERROR: Missing filename!" << endl << endl;
        usage( argc, argv );
    }

    if ( optind < argc-2 ) {
        cout << "ERROR: Too many arguments!" << endl << endl;
        usage( argc, argv );
    }

    string inputfilename( argv[optind] );
    string outputfilename( argv[optind+1] );

    /*
     *    init PIAVE 
     */
    Init();
    PluginMgr::openDLLsFromDefaultPath();


    /*
     *    open the instream
     */
    InAVFStream * instream;
    try {
        instream = StreamMgr::getAVInput( inputfilename );
    } catch ( const file_error & e ) {
        cerr << "caught PIAVE file error: "<<e.what() << endl;
        return 1;
    } catch ( const std::exception & e ) {
        cerr << "caught std exception: "<<e.what() << endl;
        return 1;
    }

    if ( instream == 0 ) {
        cout << "ERROR : no plugin can handle input format:" << endl;
        return 1;
    }

    // FIXME   make setup configurable again
    PropertyNode * is_props = instream->getProperties();
    assert( is_props );
    double fps,w,h,a;
    is_props->getAttrRec( "fps",    fps );
    is_props->getAttrRec( "width",  w );
    is_props->getAttrRec( "height", h );

    // FIXME  get aspect of screen //
    Global::renderFmt.fps     = (int) fps;    
    Global::renderFmt.width   = (int) w; 
    Global::renderFmt.height  = (int) h;

    /*
     *   try to get output plugin
     *   for output stream
     */
    OutAVStreamIFace * outstream = 0;
    PropertyNode * out_caps = new PropertyNode( "outstream" );
    PropertyNode * file = out_caps->addDaughter("file");
    PropertyNode * container = file->addDaughter("container");
    container->addAttr( "format", "rawdv" );
    // FIXME  define codec properties and look for a codec
    // check that codec are found and fit with outstream
    Plugin * p = PluginMgr::find( out_caps );
    if ( !p ) {
        WARN( "WARNING: no output plugin to handle output to file" );
    } else {
        /* reuse 'file' to create element */
        file->addAttr( "filename", outputfilename );
        PluginBase * b = p->newElement( file );
        if ( !b ) {
            WARN( "instantiating plugin "<<p->getName()<<" failed!" );
        } else {
            //  FIXME why doesn't this work ?? 
//                    outstream = dynamic_cast<OutAVStreamIFace*>( b );
            outstream = static_cast<OutAVStreamIFace*>( b );
            TRACE( "instantiating plugin "<<p->getName()<<" ok." );
            TRACE( "-------  out_caps -----------");
            TRACE( out_caps->toXML() );
            TRACE( "-------  p -----------");
            TRACE( p->getCapabilities()->toXML() );
        }
    }
    if ( outstream==0 ) {
        WARN( "no plugin can handle requested output format:");
        cout << out_caps->toXML() << endl;
        exit( 1 );
    }
    PDELETE( out_caps );
    

    /*
     *    setup render tree
     */
    AVChunk * avChunk1 = new AVChunk( instream );
    StoryBoard * story = new StoryBoard( "MasterBoard" );
    story->setHead( avChunk1 );
    if ( n_seconds < 0 || n_seconds > story->length() ) {
        n_seconds = story->length();
    }
    INFO(  PRINTV(story->length()) );


    /*
     *   main loop
     */
    PIAVE::Frame frame;
    clock_t start = clock();
    int nFrames = 0;
    for ( int i = 0 ; i< 1 ; i++ ) // multiply for testing
    {
        /*  add half a frame to start to avoid off by one errors due to rounding errors */
        for ( PIAVE::Time time = story->start() +  Global::renderFmt.getFrameLength() /2 ;
              time < n_seconds;
              time += Global::renderFmt.getFrameLength() )
        {
            nFrames++;
            NTRACE( 10, "reading frame" );
            frame = story->getFrame( time );
            NTRACE( 10, "writing frame" );
            outstream->putFrameSave( frame );
        }
    }

    clock_t stop = clock();
    double elapsed = double( stop - start );
    double unit = CLOCKS_PER_SEC/1000;
    story->printStats();


    INFO( "-----------------------------" );
    INFO( "wall time:        " 
          << elapsed/unit  << " msec" );
    INFO( "number of frames: " << nFrames );
    INFO( "frames per sec.:  " << 
          (nFrames)*1000/(elapsed/unit) );
    
    return 0;
}

