/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, Rolf.Dubitzky@gmx.net
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

// STDC
#include <unistd.h>
#include <stdio.h>
#include <getopt.h>
#include <termios.h>

// STDC++
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <string>
using std::string;
#include <cassert>

// libxml
#include <libxml/parser.h>

// PIAVE
#include "libpiave/piave_base.hh"
#include "libpiave/player.hh"
#include "libpiave/mediaelements.hh"
#include "libpiave/exception.hh"
#include "libpiave/property.hh"
#include "libpiave/plugin.hh"
#include "libpiave/pluginmgr.hh"
#include "libpiave/streammgr.hh"
#include "libpiave/bufferpool.hh"
#include "libpiave/VEML.hh"
#include "libpiave/scenebuilder.hh"
#include "libpiave/avccontroller.hh"
#include "libpiave/IEEE1394IO.hh"

using namespace PIAVE;

void usage( int argc,  char * const * argv  ) { 
    cout << "usage:  "<<argv[0]<<" [options] <file>"<<endl;
    cout << " -v <level>       level of verbosity (default: 0=quiet) " << endl;
    exit( 1 ); 
}

int
main( int argc,  char * const * argv  )
{

    string video_formatname     = "xv";
    string audio_formatname     = "oss";
    bool   default_plugin_src   = true;
    int    n_frames             = -1;
    int    n_plugins            = 0;
    double scale                = 1.0;
    double speed                = 1.0;

    int c;
    opterr = 0;
    while ((c = getopt (argc, argv, "v:h")) != -1) {
        switch (c)
        {
          case 'v':
              PIAVE::Global::verbosity = atoi( optarg );
              break;
          case 'h':
              usage( argc, argv );
              break;
          default:
              cout <<"unknown option: "<<(char)c<<endl;
              usage( argc, argv );
              break;
        }
    }

//     if ( optind == argc ) {
//         cout << "Missing filename" << endl;
//         usage( argc, argv );
//     }

//    string filename( argv[optind] );

    /*
     *    init PIAVE 
     */
    PIAVE::Init();
    PluginMgr::openDLLsFromDefaultPath();


    /*
     *   try to get screen output plugin
     */
    OutAVStreamIFace * voutstream = 0;
    OutAudioStreamIFace * aoutstream = 0;
    if ( video_formatname == "none" || true ) {
    } else {
        PropertyNode * screen_caps = new PropertyNode( "outstream" );
        PropertyNode * monitor = screen_caps->addDaughter( "monitor" );
        PropertyNode * video = monitor->addDaughter( "video" );
        video->addAttr( "type", video_formatname );
        INFO( PRINTV( video_formatname ) );
        Plugin * p = PluginMgr::find( screen_caps );
        if ( !p ) {
            cout << "WARNING: no output plugin to handle output to screen" << endl;
        } else {
            video->addAttr( "width" , Global::renderFmt.width  * scale );
            video->addAttr( "height" , Global::renderFmt.width  / Global::renderFmt.aspect * scale );
            PluginBase * b = p->newElement( video );
            if ( !b ) {
                cout << "WARNING : instantiating plugin "<<p->getName()<<endl;
            } else {
                //  FIXME why doesn't this work ?? 
//                    voutstream = dynamic_cast<OutAVStreamIFace*>( b );
                voutstream = static_cast<OutAVStreamIFace*>( b );
                if ( voutstream==0 ) {
                    cout << "WARNING : wrong plugin type " << endl;
                }
            }
        }
        PDELETE( screen_caps );
    }

    /*
     *   try to get audio output plugin
     */
    if ( audio_formatname == "none" || true ) {
    } else {
        /* define capabilities to look for */
        PropertyNode * speaker_caps = new PropertyNode( "outstream" );
        PropertyNode * monitor = speaker_caps->addDaughter( "monitor" );
        PropertyNode * audio = monitor->addDaughter( "audio" );
        audio->addAttr( "type", audio_formatname );
        /* try to get plugin */
        Plugin * p = PluginMgr::find( speaker_caps );
        if ( !p ) {
            cout << "WARNING: no output plugin to handle audio" << endl;
        } else {
            PluginBase * b = p->newElement();
            if ( !b ) {
                cout << "WARNING : instantiating plugin "<<p->getName()<<endl;
            } else {
                //  FIXME why doesn't this work ?? 
//                    voutstream = dynamic_cast<OutAVStreamIFace*>( b );
                aoutstream = static_cast<OutAudioStreamIFace*>( b );
                if ( aoutstream==0 ) {
                    cout << "WARNING : wrong plugin type " << endl;
                }
            }
        }
        PDELETE( speaker_caps );
    }

    
    /*
     *     temp hack to write to files
     */
    PropertyNode * file_caps = new PropertyNode( "outstream" );
    PropertyNode * file = file_caps->addDaughter( "file" );
    PropertyNode * container = file->addDaughter( "container" );
    container->addAttr( "format", "rawdv" );
    Plugin * p = PluginMgr::find( file_caps );
    if ( !p ) {
        cout << "WARNING: no output plugin to handle file output" << endl;
    } else {
        PropertyNode * file_props = new PropertyNode( "props" );
        file_props->addAttr( "filename", "../capture.dv" );
        PluginBase * b = p->newElement( file_props );
        if ( !b ) {
            cout << "WARNING : instantiating plugin "<<p->getName()<<endl;
        } else {
            voutstream = static_cast<OutAVStreamIFace*>( b );
            if ( voutstream==0 ) {
                cout << "WARNING : wrong plugin type " << endl;
            }
        }
        PDELETE( file_props );
    }
    PDELETE( file_caps );


    /*
     *    setup CamController
     */
    AVCController * controller;
    Raw1394Reader * reader;
    try {
        controller = AVCController::inst();
        controller->init();
        reader = Raw1394Reader::inst();
    } catch ( const exception & e ) {
        cerr << "caught std exception: "<<e.what() << endl;
        return 0;
    }


    /*
     *    setup render tree
     */
    assert( voutstream );
    Raw1394Consumer consumer;
    consumer.setOutStream( voutstream );
    // c->setOutStream( xxx );

    /*
     *    setup player
     */
//     Player player;
//     player.setStoryBoard( story );
//     player.setVideoOut( voutstream );
//     player.setAudioOut( aoutstream );
//     player.setSpeed( 2.0 );

    /*
     *    main loop
     */
    INFO( "Starting controller" );
    string help( "(h)elp -- (e)xit -- stat(u)s -- (t)imecode -- (c)apture -- st(o)p capture\n" 
                 "(p)lay -- (P)lay slow -- (r)everse -- (R)everse slow -- (s)top \n"
                 "(f)orward -- re(w)ind -- p(a)use -- (n)ext -- (q) previous" );
    cout << help << endl;
    cout << ">";
    bool quit_requested = false;
    while ( quit_requested == false && 
            controller->getState() != AVCController::error ) 
    {
        int input;
        input = getchar();
        switch ( input ) {
          case 'h':
              cout << help << endl;
              break;
          case 'e':
              quit_requested = true;
              break;
          case 'p':
              cout << "play" << endl;
              controller->vcrPlay();
              break;
          case 'P':
              cout << "play slow" << endl;
              controller->vcrPlaySlow();
              break;
          case 'r':
              cout << "reverse" << endl;
              controller->vcrReverse();
              break;
          case 'R':
              cout << "reverse slow" << endl;
              controller->vcrReverseSlow();
              break;
          case 's':
              cout << "stop" << endl;
              controller->vcrStop();
              break;
          case 'a':
              cout << "pause" << endl;
              controller->vcrPause();
              break;
          case 'f':
              cout << "forward" << endl;
              controller->vcrForward();
              break;
          case 'n':
              cout << "next" << endl;
              controller->vcrNext();
              break;
          case 'q':
              cout << "previous" << endl;
              controller->vcrPrevious();
              break;
          case 'w':
              cout << "rewind" << endl;
              controller->vcrRewind();
              break;
          case 'u':
              cout << "Camera status: " 
                   << controller->getCamStateName() << endl;
              break;
          case 't':
              cout << "Time code: " 
                   << controller->getTimeCode() << endl;
              break;
          case 'c':
              reader->startCapture();
              consumer.switchOn();
//               sleep( 1 );
//               player.switchToPlay();
//               while ( player.getState() == Player::playing ) {
//                   player.playNextFrame();
//               }
              break;
          case 'o':
              consumer.switchOff();
              reader->stopCapture();
              break;
          case '\n':
              cout << ">";
              break;
          default:
              cout << "Unknown key <"<<char(input)<<">!"<<endl;
              break;
        }
    }

    PDELETE( voutstream );

    /*
     *    print some stats
     */
    BufferPool::printStats();

    return 0;
}
